--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Italian (380) specific localizations for Netlink
--
-- See: http://www.agid.gov.it/agenda-digitale/infrastrutture-architetture/carta-nazionale-servizi/specifiche-tecniche
-- The documents on the website are licensed (http://www.agid.gov.it/note-legali) under the CC-BY 3.0 (http://creativecommons.org/licenses/by/3.0/it/)
--

require('lib.tlv')
require('netlink.l10n.380.serial_number')
require('netlink.l10n.380.carta_nazionale')

log.print(log.DEBUG, "Italian localisation loading...")

-- Parses a regional code - Italian version
function ui_parse_regional_code_localised(node, data) 
    local code=bytes.format(data,"%P")

    log.print(log.DEBUG, "Interpreting the following Italian Regional Code: " .. code)

    if #data ~= 11 then
        node:append{ classname="block", label="Unrecognized Italian format, false length", val=data}
    else
        ui_parse_region_code_localised(node, bytes.sub(data, 0, 2))

        local regional_code_raw = bytes.sub(data, 3, 10)
        local regional_code_data = string.sub(code, 4, 11)
        node:append{ classname="block", label="Codice regionale", val=regional_code_raw, alt=regional_code_data}
    end
end

-- Parses a issuer code
function ui_parse_issuer_code_localised(node, data) 
    return ui_parse_organisation_code_localised(node, data)
end

-- Parses a card issuer code
function ui_parse_card_issuer_code_localised(node, data) 
    local code=bytes.format(data,"%P")

    log.print(log.DEBUG, "Interpreting the following Italian Card Issuer Code: " .. code)

    if #data ~= 5 then
        node:append{ classname="block", label="Unrecognized Italian format, false length", val=data}
    else
        node:append{ classname="block", label="Tipo ente", val=bytes.sub(data, 0, 1), alt=bytes.format(bytes.sub(data, 0, 1), "%P")}
        ui_parse_region_code_localised(node, bytes.sub(data, 2, 4))
    end
end

-- Parses a doctor organisation code - Italian version
function ui_parse_doctor_organisation_code_localised(node, data) 
    return ui_parse_organisation_code_localised(node, data)
end

-- Parses an organisation code - Italian version
function ui_parse_organisation_code_localised(node, data) 
    local code=bytes.format(data,"%P")

    log.print(log.DEBUG, "Interpreting the following Italian Organisation Code: " .. code)

    if #data ~= 6 then
        node:append{ classname="block", label="Unrecognized Italian format, false length", val=data}
    else
        ui_parse_region_code_localised(node, bytes.sub(data, 0, 2))
        ui_parse_local_code_localised(node, bytes.sub(data, 3, 5), bytes.sub(data, 0, 2))
    end
end

-- Parses a local code - Italian version.
-- parameter regione_raw is mandatory!
function ui_parse_local_code_localised(node, data, regione_raw) 
    local code=bytes.format(data,"%P")
    local regione = nil

    if regione_raw then
        regione = bytes.format(regione_raw,"%P")
    end

    log.print(log.DEBUG, "Interpreting the following Italian Local Code: " .. code)

    if #data ~= 3 then
        node:append{ classname="block", label="Unrecognized Italian Local format, false length", val=data}
    else
        local asl = CODICI_ASL[regione .. code]

        if asl then
            local asl_node = node:append{ classname="block", label="ASL", val=data, alt=code }
            local provincia = asl[1]
            local citta = asl[2]
            local cap = asl[3]
            local nome = asl[4]

            if provincia then 
                asl_node:append{ classname="block", label="Provincia", alt=provincia }
            end
            if citta then 
                asl_node:append{ classname="block", label="Citta'", alt=citta }
            end
            if cap then 
                asl_node:append{ classname="block", label="C.A.P.", alt=cap }
            end
            if nome then 
                asl_node:append{ classname="block", label="Nome", alt=nome }
            end
        else
            log.print(log.WARNING, "Codice ASL non trovato")
            node:append{ classname="block", label="ASL", val=data, alt=code }
        end
    end
end

-- Parses a region code - Italian version
function ui_parse_region_code_localised(node, data) 
    local code=bytes.format(data,"%P")

    log.print(log.DEBUG, "Interpreting the following Italian Region Code: " .. code)

    if #data ~= 3 then
        node:append{ classname="block", label="Unrecognized Italian Region format, false length", val=data}
    else
        local asl_region_code_raw = data
        local asl_region_code_data = CODICI_ENTI_ASL[code]

        if not asl_region_code_data then
            log.print(log.WARNING, "Codice ente ASL non trovato")
            asl_region_code_data = code
        end

        node:append{ classname="block", label="Ente ASL", val=asl_region_code_raw, alt=asl_region_code_data }
    end
end


-- Regioni e provincie autonome
CODICI_ENTI_ASL = {
    ['010'] = "Regione Piemonte",
    ['020'] = "Regione Val d'Aosta",
    ['030'] = "Regione Lombardia",
    ['041'] = "Provincia di Bolzano",
    ['042'] = "Provincia di Trento",
    ['050'] = "Regione Veneto",
    ['060'] = "Regione Friuli Venezia Giulia",
    ['070'] = "Regione Liguria",
    ['080'] = "Regione Emilia Romagna",
    ['090'] = "Regione Toscana",
    ['100'] = "Regione Umbria",
    ['110'] = "Regione Marche",
    ['120'] = "Regione Lazio",
    ['130'] = "Regione Abruzzo",
    ['140'] = "Regione Molise",
    ['150'] = "Regione Campania",
    ['160'] = "Regione Puglia",
    ['170'] = "Regione Basilicata",
    ['180'] = "Regione Calabria",
    ['190'] = "Regione Sicilia",
    ['200'] = "Regione Sardegna"
    }

-- ASL, codici ente + codice locale
CODICI_ASL = {
     ['010201'] = { "TO", "TORINO", "10128", "TO1" },
     ['010202'] = { "TO", "TORINO", "10149", "TO2" },
     ['010203'] = { "TO", "COLLEGNO", "10093", "TO3" },
     ['010204'] = { "TO", "CHIVASSO", "10034", "TO4" },
     ['010205'] = { "TO", "CHIERI", "10023", "TO5" },
     ['010206'] = { "VC", "VERCELLI", "13100", "VC" },
     ['010207'] = { "BI", "BIELLA", "13900", "BI" },
     ['010208'] = { "NO", "NOVARA", "28100", "NO" },
     ['010209'] = { "VB", "OMEGNA", "28887", "VCO" },
     ['010210'] = { "CN", "CUNEO", "12100", "CN1" },
     ['010211'] = { "CN", "ALBA", "12051", "CN2" },
     ['010212'] = { "AT", "ASTI", "14100", "AT" },
     ['010213'] = { "AL", "CASALE MONFERRATO", "15033", "AL" },
     ['020101'] = { "AO", "AOSTA", "11100", "AOSTA" },
     ['030301'] = { "BG", "BERGAMO", "24121", "A.S.L. DELLA PROVINCIA DI BERGAMO" },
     ['030302'] = { "BS", "BRESCIA", "25124", "A.S.L. DELLA PROVINCIA DI BRESCIA" },
     ['030303'] = { "CO", "COMO", "22100", "A.S.L. DELLA PROVINCIA DI COMO" },
     ['030304'] = { "CR", "CREMONA", "26100", "A.S.L. DELLA PROVINCIA DI CREMONA" },
     ['030305'] = { "LC", "LECCO", "23900", "A.S.L. DELLA PROVINCIA DI LECCO" },
     ['030306'] = { "LO", "LODI", "26900", "A.S.L. DELLA PROVINCIA DI LODI" },
     ['030307'] = { "MN", "MANTOVA", "46100", "A.S.L. DELLA PROVINCIA DI MANTOVA" },
     ['030308'] = { "MI", "MILANO", "20122", "A.S.L. DI MILANO" },
     ['030309'] = { "MI", "LEGNANO", "20025", "A.S.L. DELLA PROVINCIA DI MILANO 1" },
     ['030310'] = { "MI", "MELEGNANO", "20077", "A.S.L. DELLA PROVINCIA DI MILANO 2" },
     ['030311'] = { "MB", "MONZA", "20900", "A.S.L. PROVINCIA DI MONZA E BRIANZA" },
     ['030312'] = { "PV", "PAVIA", "27100", "A.S.L. DELLA PROVINCIA DI PAVIA" },
     ['030313'] = { "SO", "SONDRIO", "23100", "A.S.L. DELLA PROVINCIA DI SONDRIO" },
     ['030314'] = { "VA", "VARESE", "21100", "A.S.L. DELLA PROVINCIA DI VARESE" },
     ['030315'] = { "BS", "BRENO", "25043", "A.S.L. DI VALLECAMONICA-SEBINO" },
     ['041201'] = { "BZ", "BOLZANO", "39100", " AZIENDA SONITARIA DELLA P.A. DI BOLZANO" },
     ['042101'] = { "TN", "TRENTO", "38100", "TRENTO" },
     ['050101'] = { "BL", "BELLUNO", "32100", "BELLUNO" },
     ['050102'] = { "BL", "FELTRE", "32032", "FELTRE" },
     ['050103'] = { "VI", "BASSANO DEL GRAPPA", "36061", "BASSANO DEL GRAPPA" },
     ['050104'] = { "VI", "THIENE", "36016", "THIENE" },
     ['050105'] = { "VI", "ARZIGNANO", "36071", "ARZIGNANO" },
     ['050106'] = { "VI", "VICENZA", "36100", "VICENZA" },
     ['050107'] = { "TV", "PIEVE DI SOLIGO", "31053", "PIEVE DI SOLIGO" },
     ['050108'] = { "TV", "ASOLO", "31011", "ASOLO" },
     ['050109'] = { "TV", "TREVISO", "31100", "TREVISO" },
     ['050110'] = { "VE", "SAN DONA' DI PIAVE", "30027", "S. DONA' DI PIAVE" },
     ['050112'] = { "VE", "VENEZIA", "30174", "VENEZIANA" },
     ['050113'] = { "VE", "MIRANO", "30035", "MIRANO" },
     ['050114'] = { "VE", "CHIOGGIA", "30015", "CHIOGGIA" },
     ['050115'] = { "PD", "CITTADELLA", "35013", "CITTADELLA" },
     ['050116'] = { "PD", "PADOVA", "35131", "PADOVA" },
     ['050117'] = { "PD", "MONSELICE", "35043", "MONSELICE" },
     ['050118'] = { "RO", "ROVIGO", "45100", "ROVIGO" },
     ['050119'] = { "RO", "ADRIA", "45011", "ADRIA" },
     ['050120'] = { "VR", "VERONA", "37122", "VERONA" },
     ['050121'] = { "VR", "LEGNAGO", "37045", "LEGNAGO" },
     ['050122'] = { "VR", "BUSSOLENGO", "37012", "BUSSOLENGO" },
     ['060101'] = { "TS", "TRIESTE", "34128", "TRIESTINA" },
     ['060102'] = { "GO", "GORIZIA", "34170", "ISONTINA" },
     ['060103'] = { "UD", "GEMONA DEL FRIULI", "33013", "ALTO FRIULI" },
     ['060104'] = { "UD", "UDINE", "33100", "MEDIO FRIULI" },
     ['060105'] = { "UD", "PALMANOVA", "33057", "BASSA FRIULANA" },
     ['060106'] = { "PN", "PORDENONE", "33170", "FRIULI OCCIDENTALE" },
     ['070101'] = { "IM", "SANREMO", "18038", "IMPERIESE" },
     ['070102'] = { "SV", "SAVONA", "17100", "SAVONESE" },
     ['070103'] = { "GE", "GENOVA", "16125", "GENOVESE" },
     ['070104'] = { "GE", "CHIAVARI", "16043", "CHIAVARESE" },
     ['070105'] = { "SP", "LA SPEZIA", "19124", "SPEZZINO" },
     ['080101'] = { "PC", "PIACENZA", "29121", "AZIENDA USL PIACENZA" },
     ['080102'] = { "PR", "PARMA", "43123", "AZIENDA USL PARMA" },
     ['080103'] = { "RE", "REGGIO NELL'EMILIA", "42122", "AZIENDA USL REGGIO EMILIA" },
     ['080104'] = { "MO", "MODENA", "41100", "AZIENDA USL MODENA" },
     ['080105'] = { "BO", "BOLOGNA", "40124", "AZIENDA USL BOLOGNA" },
     ['080106'] = { "BO", "IMOLA", "40026", "AZIENDA USL IMOLA" },
     ['080109'] = { "FE", "FERRARA", "44121", "AZIENDA USL FERRARA" },
     ['080110'] = { "RA", "RAVENNA", "48100", "AZIENDA USL RAVENNA" },
     ['080111'] = { "FC", "FORLI'", "47100", "AZIENDA USL FORLI'" },
     ['080112'] = { "FC", "CESENA", "47522", "AZIENDA USL CESENA" },
     ['080113'] = { "RN", "RIMINI", "47037", "AZIENDA USL RIMINI" },
     ['090101'] = { "MS", "CARRARA", "54033", "MASSA CARRARA" },
     ['090102'] = { "LU", "LUCCA", "55100", "LUCCA" },
     ['090103'] = { "PT", "PISTOIA", "51100", "PISTOIA" },
     ['090104'] = { "PO", "PRATO", "59100", "PRATO" },
     ['090105'] = { "PI", "PISA", "56100", "PISA" },
     ['090106'] = { "LI", "LIVORNO", "57100", "LIVORNO" },
     ['090107'] = { "SI", "SIENA", "53100", "SIENA" },
     ['090108'] = { "AR", "AREZZO", "52100", "AREZZO" },
     ['090109'] = { "GR", "GROSSETO", "58100", "GROSSETO" },
     ['090110'] = { "FI", "FIRENZE", "50122", "FIRENZE" },
     ['090111'] = { "FI", "EMPOLI", "50053", "EMPOLI" },
     ['090112'] = { "LU", "CAMAIORE", "55043", "VERSILIA" },
     ['100201'] = { "PG", "PERUGIA", "06127", "AUSL UMBRIA N. 1" },
     ['100202'] = { "TR", "TERNI", "05100", "AUSL UMBRIA N. 2" },
     ['110201'] = { "AN", "ANCONA", "60131", "ASUR" },
     ['120101'] = { "RM", "ROMA", "00185", "RM/A" },
     ['120102'] = { "RM", "ROMA", "00157", "RM/B" },
     ['120103'] = { "RM", "ROMA", "00142", "RM/C" },
     ['120104'] = { "RM", "ROMA", "00125", "RM/D" },
     ['120105'] = { "RM", "ROMA", "00193", "RM/E" },
     ['120106'] = { "RM", "CIVITAVECCHIA", "00053", "RM/F" },
     ['120107'] = { "RM", "TIVOLI", "00019", "RM/G" },
     ['120108'] = { "RM", "ALBANO LAZIALE", "00041", "RM/H" },
     ['120109'] = { "VT", "VITERBO", "01100", "VITERBO" },
     ['120110'] = { "RI", "RIETI", "02100", "RIETI" },
     ['120111'] = { "LT", "LATINA", "04100", "LATINA" },
     ['120112'] = { "FR", "FROSINONE", "03100", "FROSINONE" },
     ['130201'] = { "AQ", "L'AQUILA", "67100", "AVEZZANO-SULMONA-L'AQUILA" },
     ['130202'] = { "CH", "CHIETI", "66100", " LANCIANO-VASTO-CHIETI" },
     ['130203'] = { "PE", "PESCARA", "65100", "PESCARA" },
     ['130204'] = { "TE", "TERAMO", "64100", "TERAMO" },
     ['140201'] = { "CB", "CAMPOBASSO", "86100", " ASREM" },
     ['150201'] = { "AV", "AVELLINO", "83100", "A.S.L. AVELLINO" },
     ['150202'] = { "BN", "BENEVENTO", "82100", "A.S.L. BENEVENTO" },
     ['150203'] = { "CE", "CASERTA", "81100", "A.S.L. CASERTA" },
     ['150204'] = { "NA", "NAPOLI", "80145", "A.S.L. NAPOLI 1 CENTRO" },
     ['150205'] = { "NA", "POZZUOLI", "80078", "A.S.L. NAPOLI 2 NORD" },
     ['150206'] = { "NA", "CASTELLAMMARE DI STABIA", "80053", "A.S.L. NAPOLI 3 SUD" },
     ['150207'] = { "SA", "SALERNO", "84124", "A.S.L. SALERNO" },
     ['160106'] = { "BR", "BRINDISI", "72100", "ASL BR" },
     ['160112'] = { "TA", "TARANTO", "74100", "ASL TA" },
     ['160113'] = { "BT", "ANDRIA", "70031", "ASL BAT" },
     ['160114'] = { "BA", "BARI", "70123", " ASL BA" },
     ['160115'] = { "FG", "FOGGIA", "71100", " ASL FG" },
     ['160116'] = { "LE", "LECCE", "73100", " ASL LE" },
     ['170201'] = { "PZ", "POTENZA", "85100", "AZIENDA SANITARIA LOCALE DI POTENZA ASP" },
     ['170202'] = { "MT", "MATERA", "75100", "AZIENDA SANITARIA LOCALE DI MATERA ASM" },
     ['180201'] = { "CS", "COSENZA", "87100", " A.S.P. COSENZA" },
     ['180202'] = { "KR", "CROTONE", "88900", "A.S.P. CROTONE" },
     ['180203'] = { "CZ", "CATANZARO", "88100", "A.S.P. CATANZARO" },
     ['180204'] = { "VV", "VIBO VALENTIA", "89900", "A.S.P. VIBO VALENTIA" },
     ['180205'] = { "RC", "REGGIO DI CALABRIA", "89128", "A.S.P. REGGIO CALABRIA" },
     ['190201'] = { "AG", "AGRIGENTO", "92100", "ASP AGRIGENTO" },
     ['190202'] = { "CL", "CALTANISSETTA", "93100", "ASP CALTANISSETTA" },
     ['190203'] = { "CT", "CATANIA", "95124", "ASP CATANIA" },
     ['190204'] = { "EN", "ENNA", "94100", "ASP ENNA" },
     ['190205'] = { "ME", "MESSINA", "98100", "ASP MESSINA" },
     ['190206'] = { "PA", "PALERMO", "90100", "ASP PALERMO" },
     ['190207'] = { "RG", "RAGUSA", "97100", "ASP RAGUSA" },
     ['190208'] = { "SR", "SIRACUSA", "96100", "ASP SIRACUSA" },
     ['190209'] = { "TP", "TRAPANI", "91100", "ASP DI TRAPANI" },
     ['200101'] = { "SS", "SASSARI", "07100", "SASSARI" },
     ['200102'] = { "OT", "OLBIA", "07026", "OLBIA" },
     ['200103'] = { "NU", "NUORO", "08100", "NUORO" },
     ['200104'] = { "OG", "LANUSEI", "08045", "LANUSEI" },
     ['200105'] = { "OR", "ORISTANO", "09170", "ORISTANO" },
     ['200106'] = { "VS", "SANLURI", "09036", "SANLURI" },
     ['200107'] = { "CI", "CARBONIA", "09013", "CARBONIA" },
     ['200108'] = { "CA", "CAGLIARI", "09100", "CAGLIARI" }
    }
