--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Italian (380) specific localizations for Netlink
--
-- Specification source: http://www.agid.gov.it/agenda-digitale/infrastrutture-architetture/carta-nazionale-servizi/specifiche-tecniche
-- The documents on the original website are licensed (http://www.agid.gov.it/note-legali) under the CC-BY 3.0 (http://creativecommons.org/licenses/by/3.0/it/)
--
-- Specification of the extra filesystem: http://www.agid.gov.it/sites/default/files/documentazione_trasparenza/filesystemcns_20131216.pdf
--

require('lib.tlv')
require('netlink.l10n.380.certificate')


-- Some nation-specific scripts may want to further query the card. They will be loaded in this section.
function read_national_specifics_localised(tree)
    read_card_description_data(tree)
    read_cardholder_data(tree)
end

-- Reads the DF0 card description data tree
function read_card_description_data(tree)
    local sw, resp

    log.print(log.INFO, "Selecting the Card Description Data...")
	sw, resp = card.select("/1000/")

	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not select the Card Description Data, aborting.")
        return false
    else
        local card_description_data = tree:append{ classname="folder", label="Card Description Data", id="1000" }

        log.print(log.INFO, "Selecting Chip Data...")
	    sw, resp = card.select(".1002")

        if sw ~=0x9000 then
            log.print(log.ERROR, "Could not select the Chip Data, aborting.")
            return false
        else
            log.print(log.INFO, "Reading the Chip Data...")
            sw, resp = card.read_binary('.', 0)

            if sw ~=0x9000 then
                log.print(log.ERROR, "Could not read the Chip Data, aborting.")
                return false
            else
                local card_cd = card_description_data:append{ classname="file", label="Card Chip Data", id="1002", size=#resp, val=resp}

                if bytes.is_printable(resp) then
                    nodes.set_attribute(card_cd, 'alt', bytes.format(resp,"%P"))
                else
                    log.print(log.WARNING, "The Card Chip Data is not printable.")
                end
            end
        end

        log.print(log.INFO, "Selecting Card Serial Number...")
	    sw, resp = card.select(".1003")

        if sw ~=0x9000 then
            log.print(log.ERROR, "Could not select the Card Serial Number, aborting.")
            return false
        else
            log.print(log.INFO, "Reading the Card Serial Number...")
            sw, resp = card.read_binary('.', 0)

            if sw ~=0x9000 then
                log.print(log.ERROR, "Could not read the Card Serial Number, aborting.")
                return false
            else
                local serial_number = card_description_data:append{ classname="file", label="Card Serial Number", id="1003", size=#resp, val=resp, alt=bytes.format(resp,"%P") }
                ui_parse_actual_serial_number_localised(serial_number, bytes.format(resp,"%P"))
            end
        end
	end
end

-- Reads the DF1 cardholder data tree
function read_cardholder_data(tree)
    local sw, resp

    log.print(log.INFO, "Selecting the Cardholder Data...")
	sw, resp = card.select("/1100/")

	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not select the Cardholder Data, aborting.")
        return false
    else
        local card_description_data = tree:append{ classname="folder", label="Cardholder Data", id="1100" }

        log.print(log.INFO, "Selecting Cardholder Certificate...")
	    sw, resp = card.select(".1101")

        if sw ~=0x9000 then
            log.print(log.ERROR, "Could not select the Cardholder Certificate, aborting.")
            return false
        else
            log.print(log.INFO, "Reading the Cardholder Certificate...")
            resp = read_all_binary()

            local card_cd = card_description_data:append{ classname="file", label="Cardholder Certificate", id="1101", size=#resp}

            ui_parse_certificate(card_cd, resp)
        end

        log.print(log.INFO, "Selecting Cardholder Personal Data...")
	    sw, resp = card.select(".1102")

        if sw ~=0x9000 then
            log.print(log.ERROR, "Could not select the Cardholder Personal Data, aborting.")
            return false
        else
            log.print(log.INFO, "Reading the Cardholder Personal Data...")
            resp = read_all_binary()
            
            local personaldata = card_description_data:append{ classname="file", label="Cardholder Personal Data", id="1102", size=#resp }

            parse_cardholder_personal_data(personaldata, resp)
        end
	end
end

-- Reads the DF1 cardholder personal data
function parse_cardholder_personal_data(tree, resp)
    local resp_ascii = bytes.format(resp, "%P")

    local length = tonumber(string.sub(resp_ascii, 1, 6), 16)

    resp_ascii = string.sub(resp_ascii, 7, length-4)

    log.print(log.INFO, "Found " .. resp_ascii)

    local i=1
    local field=1

    repeat
        local nextlength  = tonumber(string.sub(resp_ascii, i, i+1), 16)

        if (nextlength > 0) then
            local desc = PERSONAL_DATA_FIELDS[field]
            local value = string.sub(resp_ascii, i+2, i + 1 + nextlength)
            local block = tree:append{ classname="block", label = desc[1], val=value}

            if desc[2] then
                desc[2](block, value)
            end
        end

        field = field+1
        i = i + 2 + nextlength        
    until i >= #resp_ascii
end

-- Parses a date in format DDMMYYYY
function ui_parse_DDMMYYYY(node, data)
	local d_table = {
	  ['year']=string.sub(data,5,8),
	  ['month']=0+string.sub(data,3,4),
	  ['day']=0+string.sub(data,1,2)}
	local t

	t = os.time(d_table)

	nodes.set_attribute(node, "val", data)
	nodes.set_attribute(node, "alt", os.date("%F",t))
end

-- Parses a gender: M=Male, F=Female
function ui_parse_gender(node, data)
    local gender = {
        ['M'] = "Male",
        ['F'] = "Female"
    }

	nodes.set_attribute(node, "alt", gender[data])
end

PERSONAL_DATA_FIELDS = {
			{ "Issuer" }, 
            { "Personalisation date", ui_parse_DDMMYYYY },
			{ "Expiration date", ui_parse_DDMMYYYY }, 
            { "Surname" },
			{ "Name" }, 
            { "Birth date", ui_parse_DDMMYYYY }, 
            { "Gender", ui_parse_gender },
			{ "Height" }, 
            { "ID code" }, 
            { "Nationality" },
			{ "Birth city" }, 
            { "Birth country" },
			{ "Birth certificate code" }, 
            { "City" },
			{ "Address" }, 
            { "Foreign validity" }
}
