--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Italian (380) specific localizations for Netlink
--
-- Specification source: http://www.agid.gov.it/agenda-digitale/infrastrutture-architetture/carta-nazionale-servizi/specifiche-tecniche
-- The documents on the original website are licensed (http://www.agid.gov.it/note-legali) under the CC-BY 3.0 (http://creativecommons.org/licenses/by/3.0/it/)
--
-- Specification of the certificate: http://www.agid.gov.it/sites/default/files/documentazione_trasparenza/strutturacertificatoautenticazionecns_v1.1_.pdf
--

require('lib.tlv')

-- Tries to parse a X.509 certificate. I am crazy.
--
-- Certificate       ::= SEQUENCE {
--   tbsCertificate      TBSCertificate,
--   signatureAlgorithm  AlgorithmIdentifier,
--   signature           BIT STRING
-- }
--
function ui_parse_certificate(node, data)
    log.print(log.DEBUG, "X.509 Trying to parse a certificate: " .. #data .. " bytes of data.")

    local tag, data, extra = asn1.split(data)

    if extra then
        log.print(log.WARNING, "X.509: Discarding " .. tostring(#extra) .. " bytes of extra data: " .. tostring(extra))
    end

    if tag == 0x30 then
        local certificate_node = node:append{ classname="item", label="X.509 Certificate", id="30", len=#data }

        log.print(log.DEBUG, "X.509 Certificate SEQUENCE found, looking for children")
        local tbs_tag, tbs_data, tbs_extra = asn1.split(data)
        if tbs_tag == 0x30 and tbs_data and tbs_extra then
            log.print(log.DEBUG, "X.509 tbsCertificate SEQUENCE found")
            local alg_tag, alg_data, alg_extra = asn1.split(tbs_extra)

            if alg_tag == 0x30 and alg_data and alg_extra then
               log.print(log.DEBUG, "X.509 signatureAlgorithm SEQUENCE found")
               local sig_tag, sig_data, sig_extra = asn1.split(alg_extra)

                if sig_tag == 0x03 and sig_data and not sig_extra then
                    log.print(log.DEBUG, "X.509 signature BIT STRING found")
                    local tbs_node = certificate_node:append{ classname="item", label="tbsCertificate", id="30", len=#tbs_data }
                    local alg_node = certificate_node:append{ classname="item", label="signatureAlgorithm", id="30", len=#alg_data }
                    local sig_node = certificate_node:append{ classname="item", label="signature", id="3", len=#sig_data, val=sig_data }
                   
                    ui_parse_tbscertificate(tbs_node, tbs_data)
                    ui_parse_signature_algorithm(alg_node , alg_data)
                else
                    log.print(log.ERROR, "X.509 Failed to parse Certificate sequence contents: signature" .. sig_tag)
                end
            else
                log.print(log.ERROR, "X.509 Failed to parse Certificate sequence contents: signatureAlgorithm")
            end
        else
            log.print(log.ERROR, "X.509 Failed to parse Certificate sequence contents: tbsCertificate")
        end
    else
        log.print(log.ERROR, "X.509 Failed to parse Certificate sequence.")
    end
end

-- Tries to parse a X.509 tbsCertificate.
-- TBSCertificate  ::=  SEQUENCE  {
--     version         [0]  Version DEFAULT v1,
--     serialNumber         CertificateSerialNumber,
--     signature            AlgorithmIdentifier,
--     issuer               Name,
--     validity             Validity,
--     subject              Name,
--     subjectPublicKeyInfo SubjectPublicKeyInfo,
--     issuerUniqueID  [1]  IMPLICIT UniqueIdentifier OPTIONAL,
--     subjectUniqueID [2]  IMPLICIT UniqueIdentifier OPTIONAL,
--     extensions      [3]  Extensions OPTIONAL
-- }
function ui_parse_tbscertificate(node, data)
    local tag, data, extra = asn1.split(data)

    if tag == 0xA0 then
        local vtag, vdata, vextra = asn1.split(data)
        if vtag == 0x02 then
            local ver_node = node:append{ classname="item", label="Version", id="A0/02", len=#vdata, val=vdata }

            ui_parse_version(ver_node, vdata)
        end
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x02 then
        local sn_node = node:append{ classname="item", label="Serial Number", id="02", len=#data, val=data }
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x30 then
        local alg_node = node:append{ classname="item", label="Signature", id="30", len=#data }

        ui_parse_signature_algorithm(alg_node , data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x30 then
        local name_node = node:append{ classname="item", label="Issuer", id="30", len=#data }

        ui_parse_name(name_node , data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x30 then
        local name_node = node:append{ classname="item", label="Validity", id="30", len=#data }

        ui_parse_validity(name_node , data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x30 then
        local name_node = node:append{ classname="item", label="Subject", id="30", len=#data }

        ui_parse_name(name_node , data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x30 then
        local name_node = node:append{ classname="item", label="Subject PublicKey Info", id="30", len=#data }

        ui_parse_publickey(name_node , data)
    end

    tlv_parse(node, extra)
end

-- Tries to parse a X.509 certificate version
--
-- Version  ::=  INTEGER  {  v1(0), v2(1), v3(2)  }
--
function ui_parse_version(node, data)
    local VERSIONS = {
         ['00'] = "v1", ['01'] = "v2", ['02'] = "v3"
    }

    local found
    if data then
        found = VERSIONS[tostring(data)]
    end

    if found then
        nodes.set_attribute(node, "alt", found)
    else
        log.print(log.WARNING, "X.509 Could not find Version number: " .. tostring(data))
    end
end

-- Tries to parse a X.509 Validity
--
-- Validity ::= SEQUENCE {
--     notBefore      Time,
--     notAfter       Time  
-- }
--
function ui_parse_validity(node, data)
    local tag, data, extra = asn1.split(data)

    if tag == 0x17 then
        local date_node = node:append{ classname="item", label="NotBefore", id="17", len=#data, val=data }

        ui_parse_printable(date_node, data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x17 then
        local date_node = node:append{ classname="item", label="NotAfter", id="17", len=#data, val=data }

        ui_parse_printable(date_node, data)
    end
end

-- Tries to parse a X.509 Public Key Info
--
-- SubjectPublicKeyInfo  ::=  SEQUENCE  {
--     algorithm            AlgorithmIdentifier,
--     subjectPublicKey     BIT STRING  
-- }
--
function ui_parse_publickey(node, data)
    local tag, data, extra = asn1.split(data)

    if tag == 0x30 then
        local alg_node = node:append{ classname="item", label="Algorithm", id="30", len=#data, val=data }

        ui_parse_signature_algorithm(alg_node, data)
    end

    tag, data, extra = asn1.split(extra)
    if tag == 0x3 then
        local pub_node = node:append{ classname="item", label="Public Key", id="03", len=#data, val=data }
    end
end

-- Tries to parse a X.509 Name
--
--   Name ::= CHOICE { -- only one possibility for now --
--     rdnSequence  RDNSequence }
--
--   RDNSequence ::= SEQUENCE OF RelativeDistinguishedName
--
--   RelativeDistinguishedName ::=
--     SET SIZE (1..MAX) OF AttributeTypeAndValue
--
--   AttributeTypeAndValue ::= SEQUENCE {
--     type     AttributeType,
--     value    AttributeValue
-- }
--
function ui_parse_name(node, data)
    local FIELDS = {
         ['30/6'] = {"AttributeType", ui_parse_algorithm_oid },
         ['30/C'] = {"AttributeValue", ui_parse_printable },
         ['31/30'] = {"AttributeTypeAndValue"},
         ['31'] = {"RelativeDistinguishedName"}
    }

    tlv_parse(node, data, FIELDS)
end

-- Tries to parse a X.509 signature algorithm.
--
-- AlgorithmIdentifier ::= SEQUENCE  {
--   algorithm             OBJECT IDENTIFIER,
--   parameters            ANY DEFINED BY algorithm OPTIONAL  }
--
function ui_parse_signature_algorithm(node, data)
    local FIELDS = {
         ['6'] = {"Algorithm Identifier", ui_parse_algorithm_oid }
    }

    tlv_parse(node, data, FIELDS)
end

-- Tries to parse a X.509 algorithm OID
function ui_parse_algorithm_oid(node, data)
    ui_parse_oid(node, data)

    local oid = nodes.get_attribute(node, "alt")

    local found = ALGORITHMS[oid]

    if found then
        nodes.set_attribute(node, "alt", found)
    else
        log.print(log.WARNING, "X.509 Could not find Algorithm OID: " .. oid)
    end
end

ALGORITHMS = {
     ['{ 1 2 840 113549 1 1 2 }'] = "md2WithRSAEncryption",
     ['{ 1 2 840 113549 1 1 4 }'] = "md5WithRSAEncryption",
     ['{ 1 2 840 113549 1 1 5 }'] = "sha-1WithRSAEncryption",
     ['{ 1 2 840 10040 4 3 }'] = "id-dsa-with-sha1",
}

VERSION_TREE = {
     ['2'] = {"Version", ui_parse_version },
}
