--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

require('netlink.topics')

-- Master File
EF_GDO = "2F02"

EF_MASTER = "3F00"

-- NETLINK AID
DF_NETLINK = "A000000073"

-- Dir listing data
EF_DIR = "2F00"

-- Card free data
EF_NKCF="D003"

-- Administrative free data
DF_NKAF="D100/"
EF_NKAF="D101"

-- Emergency free data
DF_NKEF="D200/"
EF_NKEF="D201"


-- Main function
function main()
    log.print(log.INFO, "Waiting to connect to a card...")
	if card.connect() then
	    operate_on_connected_card()

        log.print(log.INFO, "Disconnecting from the card.")
	    card.disconnect()
    else
        log.print(log.ERROR, "Card connection failed, aborting.")
    end
end

-- Operates on the newly connected card
function operate_on_connected_card()
    local tree;

    tree = card.tree_startup("Healthcare card (Netlink)")

    read_master_file(tree)
    read_netlink_application(tree)
    read_national_specifics(tree)
end


-- After the ATR, the MF is implicitly read. ICCSN and CHN (cardholder name) can be read immediately.
function read_master_file(tree)
    local sw, resp

    log.print(log.INFO, "Selecting the master application...")
	sw, resp = card.select("#" .. EF_MASTER)

    local master_file = tree:append{ classname="folder", label="Master File", id="3F00" }

    log.print(log.INFO, "Selecting the Global Data Objects...")
	sw, resp = card.select("." .. EF_GDO)

	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not select the Global Data Objects, aborting.")
        return false
    else

        log.print(log.INFO, "Reading the Global Data Objects...")
        sw, resp = card.read_binary('.', 0)

        if sw ~=0x9000 then
            log.print(log.ERROR, "Could not read the Global Data Objects, aborting.")
            return false
        else
            local global_data_objects = master_file:append{ classname="file", label="Global Data Objects", id=EF_GDO, size=#resp }
            decode_global_data_objects(global_data_objects, resp)
        end
	end
end

-- The netlink application can be directly selected
function read_netlink_application(tree)
    local master_file, global_data_objects, sw, resp

    log.print(log.INFO, "Selecting the Netlink application...")
	sw, resp = card.select("#" .. DF_NETLINK)

	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not open the Netlink Application, aborting.")
        return false
    else
        local netlink_application = tree:append{ classname="application", label="Netlink Application", id=DF_NETLINK }

        read_directory_data(netlink_application)
        read_card_free_data(netlink_application)
        read_free_administrative_data(netlink_application)
        read_free_emergency_data(netlink_application)
	end
end

-- The Netlink Directory Data file can be directly selected under the netlink application
function read_directory_data(tree)
    local sw, resp

    sw, resp = card.select("#" .. DF_NETLINK)

    log.print(log.INFO, "Selecting the Directory Data file...")
	sw, resp = card.select("."..EF_DIR)
	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not open the Directory Data file, aborting.")
        return false
    else
        local directory = tree:append{ classname="file", label="Directory Data", id=EF_DIR }

        log.print(log.INFO, "Reading the Netlink Directory Data File...")

        resp = read_all_binary()
        ui_parse_dir_structure(directory, resp)
	end
end

-- The Netlink Card Free Data file can be directly selected under the netlink application
function read_card_free_data(tree)
    local sw, resp

    sw, resp = card.select("#" .. DF_NETLINK)

    log.print(log.INFO, "Selecting the Card Free Data file...")
	sw, resp = card.select("."..EF_NKCF)
	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not open the Card Free Data file, aborting.")
        return false
    else
        local card_free = tree:append{ classname="file", label="Card Free Data", id=EF_NKCF }

        log.print(log.INFO, "Reading the Netlink Card Free Data File...")

        resp = read_all_binary()
        decode_card_free_data(card_free, resp)
	end
end

-- The Netlink Administrative Free Data file can be directly selected under the netlink application
function read_free_administrative_data(tree)
    local sw, resp

    sw, resp = card.select("#" .. DF_NETLINK)

    log.print(log.INFO, "Selecting the Netlink Administrative Free Data file...")
	sw, resp = card.select("."..DF_NKAF)
    sw, resp = card.select("."..EF_NKAF)
	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not open the Netlink Administrative Free Data file, aborting.")
        return false
    else
        local free_administrative = tree:append{ classname="file", label="Administrative Free Data", id=DF_NKAF..EF_NKAF }

        log.print(log.INFO, "Reading the Netlink Administrative Free Data File...")

        resp = read_all_binary()
        decode_free_administrative_data(free_administrative, resp)
	end
end

-- The Netlink Emergency Free Data file can be directly selected under the netlink application
function read_free_emergency_data(tree)
    local sw, resp

    sw, resp = card.select("#" .. DF_NETLINK)

    log.print(log.INFO, "Selecting the Netlink Emergency Free Data file...")
	sw, resp = card.select("."..DF_NKEF)
    sw, resp = card.select("."..EF_NKEF)
	if sw ~=0x9000 then
        log.print(log.ERROR, "Could not open the Netlink Emergency Free Data file, aborting.")
        return false
    else
        local free_emergency = tree:append{ classname="file", label="Emergency Free Data", id=DF_NKEF..EF_NKEF }

        log.print(log.INFO, "Reading the Netlink Emergency Free Data File...")

        resp = read_all_binary()
        decode_free_emergency_data(free_emergency, resp)
	end
end

-- Some nation-specific scripts may want to further query the card. They will be loaded in this section.
function read_national_specifics(tree)
    if read_national_specifics_localised then
        read_national_specifics_localised(tree)
    else
        log.print(log.WARNING, "No nation-specific extension to Netlink cards, or nation changes not implemented.")
    end
end
