--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

require('lib.tlv')
require('netlink.util')
require('netlink.l10n')
require('netlink.enums')

-- Parses the name details
function ui_parse_name_details(tree, data)
    local NAME_DETAILS = {
         ['80'] = {"Title", ui_parse_printable },
         ['81'] = {"Surname Prefix", ui_parse_printable },
         ['82'] = {"Given surname", ui_parse_printable },
         ['A3'] = {"Alternative surnames"},
         ['A3/4'] = {"Alternative surname", ui_parse_printable },
         ['84'] = {"Surname Suffix", ui_parse_printable },
         ['A5'] = {"First names"},
         ['A5/4'] = {"First name", ui_parse_printable },
         ['86'] = {"Preferred name", ui_parse_printable },
         ['87'] = {"Birth surname", ui_parse_printable },
    }

    tlv_parse(tree, data, NAME_DETAILS)
end

-- Parses the birth details
function ui_parse_birth_details(tree, data)
    local BIRTH_DETAILS = {
         ['80'] = {"Date", ui_parse_ASCII_YYYYMMDD },
         ['81'] = {"Gender", ui_parse_gender },
         ['82'] = {"Birth location", ui_parse_ASCII_country_code },
    }

    tlv_parse(tree, data, BIRTH_DETAILS)
end

-- Parses the organ donor details
function ui_parse_organ_donor_details(tree, data)
    local DONOR_DETAILS = {
         ['80'] = {"Organ category", ui_parse_organ_category },
         ['81'] = {"Choice", ui_parse_organ_donor_choice },
    }

    tlv_parse(tree, data, DONOR_DETAILS)
end

-- Parses the card update details
function ui_parse_update_details(tree, data)
    local UPDATE_DETAILS = {
         ['80'] = {"Date", ui_parse_ASCII_YYYYMMDD },
         ['A1'] = {"Update responisble" },
         ['A1/80'] = {"Country", ui_parse_ASCII_country_code },
         ['A1/81'] = {"HPC Serial Number", ui_parse_printable },
         ['A1/82'] = {"Name", ui_parse_printable },
    }

    tlv_parse(tree, data, UPDATE_DETAILS)
end


-- Parses the patient identification
function ui_parse_patient_identification(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        if tag == 0x31 then
            local PATIENT_IDENTIFICATION = {
                 ['81'] = {"Identification Code", ui_parse_printable },
                 ['A0'] = {"Issuer"},
                 ['A0/80'] = {"Sector code", ui_parse_printable },
                 ['A0/81'] = {"Nation code", ui_parse_ASCII_country_code },
                 ['A0/82'] = {"Issuer code", ui_parse_card_issuer_code },
                 ['A0/83'] = {"Checksum"}
            }

            tlv_parse(tree, data, PATIENT_IDENTIFICATION)
        else
            log.print(log.WARNING, "Patient identification tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses many administrative details
function ui_parse_administrative_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        if tag == 0x31 then
            ui_parse_single_administrative_details(tree, data)
        else
            log.print(log.WARNING, "Administrative details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses many addresses details
function ui_parse_address_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        if tag == 0x31 then
            ui_parse_single_address_details(tree, data)
        else
            log.print(log.WARNING, "Address details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses many doctors details
function ui_parse_doctors_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        if tag == 0x31 then
            ui_parse_single_doctors_details(tree, data)
        else
            log.print(log.WARNING, "Family doctors details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses one administrative unit's details
function ui_parse_single_administrative_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        local tag_string = string.format("%X", tag)

        if tag == 0x80 then
            ui_parse_ASCII_country_code(tree:append{ classname="item", label="Country", id=tag_string, count=#data }, data)
        elseif tag == 0x81 then
            ui_parse_issuer_code(tree:append{ classname="item", label="Issuer code", id=tag_string, count=#data }, data)
        elseif tag == 0x82 then
            ui_parse_printable(tree:append{ classname="item", label="Issuer name", id=tag_string, count=#data }, data)
        elseif tag == 0xA3 then
            ui_parse_single_address_structure(tree:append{ classname="item", label="Address structure", id=0xA2, count=#data }, data)
        elseif tag == 0xA4 then
            ui_parse_single_communication_structure(tree:append{ classname="item", label="Communication structure", id=0xA3, count=#data }, data)
        elseif tag == 0xA6 then
            ui_parse_insurance_structure(tree:append{ classname="item", label="Insurance codes", id=tag_string, count=#data }, data)
        elseif tag == 0xA8 then
            ui_parse_validity_structure(tree:append{ classname="item", label="Card validity period", id=tag_string, count=#data }, data)
        elseif tag == 0xB8 then
            ui_parse_validity_structure(tree:append{ classname="item", label="Initial card validity period", id=tag_string, count=#data }, data)
        else
            log.print(log.WARNING, "Single Administrative details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses one doctors details
function ui_parse_single_doctors_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        local tag_string = string.format("%X", tag)

        if tag == 0x80 then
            tree:append{ classname="item", label="Surname, Name", val=data, alt=bytes.format(data,"%P"), id=tag_string, count=#data }
        elseif tag == 0x81 then
            ui_parse_doctor_qualification(tree:append{ classname="item", label="Qualifications", id=tag_string, count=#data }, data)
        elseif tag == 0x84 then
            tree:append{ classname="item", label="Regional doctor code", val=data, alt=bytes.format(data,"%P"), id=tag_string, count=#data }
        elseif tag == 0xA2 then
            ui_parse_single_address_structure(tree:append{ classname="item", label="Address structure", id=0xA2, count=#data }, data)
        elseif tag == 0xA3 then
            ui_parse_single_communication_structure(tree:append{ classname="item", label="Communication structure", id=0xA3, count=#data }, data)
        elseif tag == 0xB5 then
            ui_parse_single_otherinfo_structure(tree:append{ classname="item", label="Other information", id=tag_string, count=#data }, data)
        else
            log.print(log.WARNING, "Family doctors details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses one address details
function ui_parse_single_address_details(tree, data)
    local tag, extra

    repeat
        tag, data, extra = asn1.split(data)

        local tag_string = string.format("%X", tag)

        if tag == 0x80 then
            tree:append{ classname="item", label="Address type", val=data, alt=bytes.tonumber(data), id=tag_string, count=#data }
        elseif tag == 0xA1 then
            ui_parse_single_address_structure(tree:append{ classname="item", label="Address structure", id=tag_string, count=#data }, data)
        elseif tag == 0xA2 then
            ui_parse_single_communication_structure(tree:append{ classname="item", label="Communication structure", id=tag_string, count=#data }, data)
        else
            log.print(log.WARNING, "Address details tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil
end

-- parses one address structure
function ui_parse_single_address_structure(tree, data)
    local STRUCTURE = {
         ['A0'] = {"Addresses"},
         ['A0/4'] = {"Address", ui_parse_printable },
         ['81'] = {"Zip code", ui_parse_printable },
         ['82'] = {"Country", ui_parse_ASCII_country_code }
    }

    tlv_parse(tree, data, STRUCTURE)
end

-- parses a card issuer structure
function ui_parse_card_issuer_structure(tree, data)
    local STRUCTURE = {
         ['80'] = {"Major Industry Identifier", ui_parse_printable},
         ['81'] = {"Country", ui_parse_ASCII_country_code },
         ['82'] = {"Issuer Identifier", ui_parse_card_issuer_code },
         ['83'] = {"Check Digit"}
    }

    tlv_parse(tree, data, STRUCTURE)
end

-- parses one insurance structure
function ui_parse_insurance_structure(tree, data)
    local STRUCTURE = {
         ['80'] = {"Regional insurance code", ui_parse_regional_code }
    }

    tlv_parse(tree, data, STRUCTURE)
end


-- Decodes the GDO value
function decode_global_data_objects(tree, resp)
    local EF_GDO_REFERENCE = {
        ['5A'] = {"ICC Serial Number", ui_parse_iccsn },
        ['5F20'] = {"Cardholder Name", ui_parse_printable },
        ['53'] = {"Discretionary data", ui_parse_discretionary_data  }
    }

    tlv_parse(tree, resp, EF_GDO_REFERENCE)
end

-- parses one doctor's other info structure
function ui_parse_single_otherinfo_structure(tree, data)
    local STRUCTURE = {
         ['80'] = {"Doctor organisation code", ui_parse_doctor_organisation_code },
         ['81'] = {"Regional substitute code", ui_parse_printable },
         ['82'] = {"Doctor assignment expiration", ui_parse_ASCII_YYYYMMDD },
         ['83'] = {"Healthcare expiration", ui_parse_ASCII_YYYYMMDD },
         ['84'] = {"Version", ui_parse_printable },
    }

    tlv_parse(tree, data, STRUCTURE)
end

-- parses one directory structure
function ui_parse_dir_structure(tree, data)
    local DIR_REFERENCE = {
        ['61'] = {"Card Application Identification" },
        ['4F'] = {"Application Identifier" },
        ['73'] = {"Discretionary Data"},
        ['73/80'] = {"Card Application Type", ui_parse_card_application_type },
        ['73/81'] = {"Card Application Version", ui_parse_printable  }
    }

    tlv_parse(tree, data, DIR_REFERENCE)
end


-- parses one communication structure
function ui_parse_validity_structure(tree, data)
    local STRUCTURE = {
         ['80'] = {"Start", ui_parse_ASCII_YYYYMMDD },
         ['81'] = {"End", ui_parse_ASCII_YYYYMMDD }
    }

    tlv_parse(tree, data, STRUCTURE)
end

-- parses one communication structure
function ui_parse_single_communication_structure(tree, data)
    local STRUCTURE = {
         ['A0'] = {"Telephone numbers"},
         ['A0/12'] = {"Telephone number", ui_parse_printable },
         ['81'] = {"Fax number", ui_parse_printable },
         ['82'] = {"Email address", ui_parse_printable }
    }

    tlv_parse(tree, data, STRUCTURE)
end

-- Parses an address set
function ui_parse_address_set(node, data) 
    local address = nil
    local tag, extra

	nodes.set_attribute(node,"val",data)

	if #data==0 then
		return false
	end

    repeat
        tag, data, extra = asn1.split(data)

        if tag == 0x4 then
            address = address .. bytes.format(data,"%P")
        else
            log.print(log.WARNING, "Address set tag " .. tag .. " unknown.")
        end

        data = extra
    until data==nil

	if address then
		nodes.set_attribute(node, "alt", address)
    end
end
