--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

require('lib.tlv')
require('netlink.util')
require('netlink.subtopics')

-- Decodes the Card Free Data
function decode_card_free_data(tree, resp)
    if resp then
        log.print(log.INFO, "Decoding Card Free Data.")
    else
        log.print(log.ERROR, "No Emergency Free Data found, aborting.")
        return false
    end

    local EF_NKCF_SUBPROCESSORS = {
        ['61'] = {"Card Application Identification", ui_parse_dir_structure},
        ['81'] = {"Card Holder Identifier", ui_parse_printable},
        ['82'] = {"Card Identifier", ui_parse_printable},
        ['83'] = {"Card Status", ui_parse_card_status },
        ['A0'] = {"Card Issuer", ui_parse_card_issuer_structure},
    }

    local tag, data, extra = asn1.split(resp)

    if extra then
        log.print(log.WARNING, "Following the ASN.1 data length, discarding " .. tostring(#extra) .. " bytes of data: " .. tostring(extra))
    end

    if tag == 0x31 then
        repeat
            log.print(log.INFO, "Decoding " .. #data .. " bytes of Card Free data...")
            tag, data, extra = asn1.split(data)
            
            local tag_string = string.format("%X", tag)

            local lookup_record = EF_NKCF_SUBPROCESSORS[tag_string]
            local sub_node, parser, description="Unknown" 

            if lookup_record then
                description = lookup_record[1]
                parser = lookup_record[2]

	            sub_node = nodes.append(tree,{ classname = "item", label = description, id = tag_string, size = #data } )
            end

            if parser then
                log.print(log.INFO, "Card Free Data Tag " .. tag_string .. " (" .. tostring(description) .. ") is going to be parsed.")
                parser(sub_node, data)
            else
                log.print(log.ERROR, "Tag " .. tag_string .. " (" .. tostring(description) .. ") not implemented, discarding data: " .. tostring(data))
            end

            log.print(log.INFO, "Proceed " .. tostring(tag) .. ", " .. tostring(data) .. ", " .. tostring(extra))
            data = extra
        until tag == nil or data == nil
    else
        log.print(log.ERROR, "Expected tag 31, got instead " .. tostring(tag))
        return false
    end
end

-- Decodes the Free Administrative Data
function decode_free_administrative_data(tree, resp)
    if resp then
        log.print(log.INFO, "Decoding Administrative Free Data.")
    else
        log.print(log.ERROR, "No Administrative Free Data found, aborting.")
        return false
    end

    local EF_NKAF_SUBPROCESSORS = {
        ['A0'] = {"Patient Identification", ui_parse_patient_identification},
        ['A1'] = {"Name Details", ui_parse_name_details},
        ['A2'] = {"Languages", ui_parse_language_details},
        ['A3'] = {"Birth", ui_parse_birth_details},
        ['A4'] = {"Addresses", ui_parse_address_details},
        ['A5'] = {"Contacts", ui_parse_contact_details},
        ['A6'] = {"Administrative Details", ui_parse_administrative_details},
        ['A7'] = {"Assigned Doctor", ui_parse_doctors_details},
        ['A8'] = {"Organ Donor Details", ui_parse_organ_donor_details},
        ['A9'] = {"Last Update", ui_parse_update_details},
        ['B1'] = {"First Personalisation", ui_parse_update_details},
    }

    local tag, data, extra = asn1.split(resp)

    if extra then
        log.print(log.WARNING, "Following the ASN.1 data length, discarding " .. tostring(#extra) .. " bytes of data: " .. tostring(extra))
    end

    if tag == 0x31 then
        repeat
            log.print(log.INFO, "Decoding " .. #data .. " bytes of Administrative data...")
            tag, data, extra = asn1.split(data)
            
            local tag_string = string.format("%X", tag)

            local lookup_record = EF_NKAF_SUBPROCESSORS[tag_string]
            local sub_node, parser, description="Unknown" 

            if lookup_record then
                description = lookup_record[1]
                parser = lookup_record[2]

	            sub_node = nodes.append(tree,{ classname = "item", label = description, id = tag_string, size = #data } )
            end

            if parser then
                log.print(log.INFO, "Administrative Free Data Tag " .. tag_string .. " (" .. tostring(description) .. ") is going to be parsed.")
                parser(sub_node, data)
            else
                log.print(log.ERROR, "Tag " .. tag_string .. " (" .. tostring(description) .. ") not implemented, discarding data: " .. tostring(data))

                if sub_node ~= nil then
                    sub_node:set_attribute('alt', "Parser not yet implemented")
                    sub_node:set_attribute('val', tostring(data))
                end
            end

            log.print(log.INFO, "Proceed " .. tostring(tag) .. ", " .. tostring(data) .. ", " .. tostring(extra))
            data = extra
        until tag == nil or data == nil
    else
        log.print(log.ERROR, "Expected tag 31, got instead " .. tostring(tag))
        return false
    end
end

-- Decodes the Free Emergency Data
function decode_free_emergency_data(tree, resp)
    if resp then
        log.print(log.INFO, "Decoding Emergency Free Data.")
    else
        log.print(log.ERROR, "No Emergency Free Data found, aborting.")
        return false
    end

    local tag, data, extra = asn1.split(resp)

    if extra then
        log.print(log.WARNING, "Following the ASN.1 data length, discarding " .. tostring(#extra) .. " bytes of data: " .. tostring(extra))
    end

    if tag == 0x31 then
        log.print(log.INFO, "Decoding " .. #data .. " bytes of Emergency data...")
        log.print(log.WARNING, "Implementation missing: sorry, this part has not been yet implemented.")
        tlv_parse(tree, data, {})
    else
        log.print(log.ERROR, "Expected tag 31, got instead " .. tostring(tag))
        return false
    end
end
