--
--      Copyright 2015 Andrea Micheloni <a4a@m7i.org>
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--

require('lib.tlv')
require('netlink.l10n')
require('netlink.enums')

-- Parses the card Serial Number
function ui_parse_iccsn(tree, data)
    log.print(log.DEBUG, "Interpreting the following ICC Serial Number: " .. tostring(data))

    if #data ~= 14 then
        tree:append{ classname="block", label="Unrecognized format, false length", val=data}
    else
        local country_code_data = string.sub(tostring(bytes.sub(data, 1, 2)), 1, 3)
        local country_code = tree:append{ classname="block", label="Country Code" }
        ui_parse_country_code(country_code, country_code_data)
        load_country_l10n(country_code_data)

        local issuer_id_data = string.sub(tostring(bytes.sub(data, 2, 5)), 2, 6)
        local issuer_id = tree:append{ classname="block", label="Issuer ID", val=issuer_id_data }
        ui_parse_card_issuer_code(issuer_id, bytes.new_from_chars(issuer_id_data))

        local serial_number_data_raw = bytes.sub(data, 5, 12)
        local serial_number = tree:append{ classname="block", label="Serial Number", val=serial_number_data_raw, alt=tostring(serial_number_data_raw)}
        ui_parse_actual_serial_number(serial_number, serial_number_data_raw)
    end
end

-- Parses the Discretionary Data
function ui_parse_discretionary_data(tree, data)
    log.print(log.DEBUG, "Interpreting the following Discretionary Data: " .. tostring(data))

    if #data ~= 27 then
        tree:append{ classname="block", label="Unrecognized format, false length", val=data}
    else
        local card_type_data_raw = bytes.sub(data, 0, 2)
        local card_type_data = bytes.format(card_type_data_raw, "%P")
        local card_type = tree:append{ classname="block", label="Card Type", val=card_type_data_raw, alt=card_type_data }

        local fs_version_data_raw = bytes.sub(data, 3, 6)
        local fs_version_data = bytes.format(fs_version_data_raw, "%P")
        local fs_version = tree:append{ classname="block", label="File System Version", val=fs_version_data_raw, alt=fs_version_data }

        for i=7, 26, 4 do
            local ef_raw = bytes.sub(data, i, i+3)
            local ef_name = tostring(bytes.sub(data, i, i+1))
            local ef_length = bytes.tonumber(bytes.sub(data, i+2, i+3))
            local ef = tree:append{ classname="block", label="Elementary File length", val=ef_raw, alt="File '" .. ef_name .. "' is " .. ef_length .. " bytes long" }
        end
    end
end

-- Parses a date in format YYYYMMDD ascii coded
function ui_parse_ASCII_YYYYMMDD(node, data)
    local d = bytes.format(data,"%P")
	local d_table = {
	  ['year']=string.sub(d,1,4),
	  ['month']=0+string.sub(d,5,6),
	  ['day']=0+string.sub(d,7,8)}
	local t

	t = os.time(d_table)

	nodes.set_attribute(node, "val", data)
	nodes.set_attribute(node, "alt", os.date("%F",t))
end

-- Parses a country code in format YYYYMMDD ascii coded
function ui_parse_ASCII_country_code(node,data)
    local cc_name = iso_country_code_name(tonumber(bytes.format(data, "%P")))

	nodes.set_attribute(node,"val",data)
	nodes.set_attribute(node,"alt",cc_name)
	return true
end

-- Reads a binary until receiving a status different from 0x9000, reading less than 256 bytes or having an error
function read_all_binary()
    local sw, resp
    local pos = 0
    local data = bytes.new(8)

    repeat 
	    sw, resp = card.read_binary('.',pos)
	    if resp then
		    pos = pos + #resp
		    data = data .. resp
	    end
    until sw~=0x9000 or resp==nil or #resp<256

    return data
end 
